/*  xb_test_xbase.cpp

XBase64 Software Library

Copyright (c) 1997,2003,2014,2022,2023 Gary A Kunkel

The xb64 software library is covered under the terms of the GPL Version 3, 2007 license.

Email Contact:

    XDB-devel@lists.sourceforge.net
    XDB-users@lists.sourceforge.net

*/

// This program tests the class xbXBase

// usage:   xb_test_xbase QUITE|NORMAL|VERBOSE


#include "xbase.h"

using namespace xb;

#include "tstfuncs.cpp"

int main( int argCnt, char **av )
{
  int iRc = 0;
  int iPo = 1;          /* print option */
                       /* 0 - QUIET    */
                       /* 1 - NORMAL   */
                       /* 2 - VERBOSE  */

  if( argCnt > 1 ) {
    if( av[1][0] == 'Q' )
      iPo = 0;
    else if( av[1][0] == 'V' )
      iPo = 2;
  }

  xbXBase x;
  xbString sMsg;

  #ifdef XB_LOGGING_SUPPORT


  xbString sLogDir = PROJECT_LOG_DIR;
//  x.SetLogDirectory( sLogDir );
  x.SetLogDirectory( PROJECT_LOG_DIR );


  x.EnableMsgLogging();
  if( iPo ){
    std::cout << "Logfile is [" << x.GetLogFqFileName().Str() << "]" << std::endl;
  }

  sMsg.Sprintf( "Program [%s] initializing...", av[0] );
  x.WriteLogMessage( sMsg );
  #endif
  InitTime();


  x.SetDefaultDateFormat( "YY-MM-DD" );
  iRc += TestMethod( iPo, "SetDefaultDateFormat", x.GetDefaultDateFormat(), "YY-MM-DD", 8 );
  if( iPo == 2 ){
    if( x.GetEndianType() == 'L' )
      std::cout << "Little Endian Architecture" << std::endl;
    else if( x.GetEndianType() == 'B' )
      std::cout << "Big Endian Architecture" << std::endl;
    else
      std::cout << "Undefine Endian Architecture" << std::endl;
  }

  iRc += TestMethod( iPo, "GetErrorMessage", x.GetErrorMessage( XB_DBF_FILE_NOT_OPEN ), "DBF File Not Open", 17 );


  #ifdef XB_LOGGING_SUPPORT
  iRc += TestMethod( iPo, "GetLogDirectory()", x.GetLogDirectory(), sLogDir, sLogDir.Len());
  xbString sLogName = CMAKE_SYSTEM_NAME;
  sLogName += "_";
  sLogName += XB_PLATFORM;
  sLogName += ".xbLog.txt";
  iRc += TestMethod( iPo, "GetLogFileName()", x.GetLogFileName(), sLogName, sLogName.Len());
  x.WriteLogMessage( "Program xb_test_xbase - test logfile message" );
  #endif

  x.xbSleep( 250 );
  if( iPo == 2 ){
    std::cout << "DisplayError Test ==> ";
    x.DisplayError( 0 );
  }
  if( iPo > 0 || iRc < 0 )
    fprintf( stdout, "Total Errors = %d\n", iRc * -1 );


  #ifdef XB_LOGGING_SUPPORT
  sMsg.Sprintf( "Program [%s] terminating with [%d] errors...", av[0], iRc * -1 );
  x.WriteLogMessage( sMsg );
  #endif

  return iRc;
}

