/*  xb_dumprecs.cpp

XBase64 Software Library

Copyright (c) 1997,2003,2014,2019,2023 Gary A Kunkel

The xb64 software library is covered under 
the terms of the GPL Version 3, 2007 license.

Email Contact:

    xb64-devel@lists.sourceforge.net
    xb64-users@lists.sourceforge.net

*/

#include <xbase.h>
using namespace xb;

void PrintHelp();
void PrintHelp(){
  std::cout << "Usage: xb_dumprecs [-h] [-?] [--help] [-v] [--version] -i filename.DBF"  << std::endl << std::endl;
  std::cout << "This program dumps data records in an xbase formatted DBF file to stdout in csv format." << std::endl;
  std::cout << "Memo data (variable length) is not included in this export.  See program xb_dumpdpt for memo data." << std::endl << std::endl;
}
void PrintVersion();
void PrintVersion(){
  std::cout << "Xbase64 Version: " << xbase_VERSION_MAJOR << "." << xbase_VERSION_MINOR << "." << xbase_VERSION_PATCH << std::endl;
}

int main(int argc, char *argv[] )
{
  xbXBase x;


  xbInt16 iRc;
  x.EnableMsgLogging();
  x.SetLogSize( 1000000L );

  xbString sParm;
  if (argc < 2 || x.GetCmdLineOpt( argc, argv, "-h", sParm ) ||
                  x.GetCmdLineOpt( argc, argv, "-?", sParm ) ||
                  x.GetCmdLineOpt( argc, argv, "--help", sParm )){
    PrintHelp();
    return 1;
  }

  if ( x.GetCmdLineOpt( argc, argv, "-v", sParm ) ||
       x.GetCmdLineOpt( argc, argv, "--version", sParm )){
    PrintVersion();
    return 1;
  }

  if( !x.GetCmdLineOpt( argc, argv, "-i", sParm ) || sParm == "" ){
    PrintHelp();
    return 1;
  }

  xbDbf *MyFile = NULL;
  if(( iRc = x.OpenHighestVersion( sParm.Str(), "", &MyFile )) != XB_NO_ERROR ){
    std::cout << "Could not open file RC = " << iRc  << " file = "  << sParm.Str() << std::endl;
    x.DisplayError( iRc );
    return 1;
  }

  //  std::cout << "Processing file sequentially from beginning..." << std::endl;

  #ifdef XB_BLOCKREAD_SUPPORT
  MyFile->EnableBlockReadProcessing();
  #endif

  xbUInt32 j = 0;
  xbUInt32 ulRecCnt = 0;

  iRc = MyFile->GetRecordCnt( ulRecCnt );

  if( iRc < XB_NO_ERROR )
    return iRc;
  while( j < ulRecCnt ){
    if( j == 0 )
      iRc = MyFile->DumpRecord(++j, 2, 2 );   // header + data
    else
      iRc = MyFile->DumpRecord(++j, 2, 1 );   // data only
    if( iRc != XB_NO_ERROR ){
      x.DisplayError( iRc );
      return 1;
    }
  }
  std::cout << j << " Records processed." << std::endl;

  #ifdef XB_BLOCKREAD_SUPPORT
  MyFile->DisableBlockReadProcessing();
  #endif

  MyFile->Close();

  return 0;
}
